import numpy as np
import SimpleITK as sitk

class MicrostructureImage:

    def __init__(self):
        self._colors = np.zeros((1,1,1))
        self.colorsRead = False
        self.res = 1.

    # Properties are analogous to getter methods in other languages
    # Note: These are accessed like attributes, e.g., MicrostructureImage.colors instead of MicrostructureImage.colors()
    @property
    def colors(self):
        # Access the microstructure (if read) as a numpy array
        # Every material is associated with a certain integer (its color)
        # Example: MicrostructureImage.colors[1,2,3] gives the integer (the color) of the material at coordinates (1,2,3)

        if self.colorsRead:
            return self._colors
        else:
            raise Exception("Colors have not been read, yet!")

    def readColors(self, filename):
        # Reading the microstructure

        print(f"input filename: {filename}")

        inputImage = sitk.ReadImage(filename)

        spacing = np.array(inputImage.GetSpacing())

        tol = 1e-6
        equalSpacing = np.all(abs(spacing - spacing[0]) <= tol*spacing[0])
        assert equalSpacing, "Spacing must be equal in all dimensions!"

        self._colors = sitk.GetArrayFromImage(inputImage)
        del inputImage

        assert self._colors.dtype=='uint8', f"Array has wrong data type {self._colors.dtype}, should be uint8"
        assert self._colors.ndim==3, f"Array has wrong dimension of {self._colors.ndim}, should be 3"

        self._colors = self._colors.T.copy()

        print(f"Dimensions :{np.shape(self._colors)}")

        self.colorsRead = True


