###############################################################################
###                                                                         ###
###                        WRITE FIELD OUTPUT                               ###
###                                                                         ###
###############################################################################

### Please load this class in your main class via, e.g.,
### from Image.WriteOutput import PyEVTKOutput

class PyEVTKOutput:

    def __init__(self, folder):
        self.folder = folder
        self.myCellData = {}

    def addScalarField(self, array, tag):
        my_array = array.copy()
        self.myCellData.update({tag: my_array})

    def addMaterials(self, array):
        self.addScalarField(array, 'Materials')

    def addVectorField(self, array, tag):
        raise Exception('addVectorField no available for PyEVTKOutput')

    def addTensor6Field(self, array, tag):
        raise Exception('addTensor6Field no available for PyEVTKOutput')

    def addStress(self, field):
        self.addScalarField(field[0,:,:,:], 'Stress_11')
        self.addScalarField(field[1,:,:,:], 'Stress_22')
        self.addScalarField(field[2,:,:,:], 'Stress_33')
        self.addScalarField(field[3,:,:,:], 'Stress_23')
        self.addScalarField(field[4,:,:,:], 'Stress_13')
        self.addScalarField(field[5,:,:,:], 'Stress_12')

    def addStrain(self, field):
        self.addScalarField(field[0,:,:,:], 'Strain_11')
        self.addScalarField(field[1,:,:,:], 'Strain_22')
        self.addScalarField(field[2,:,:,:], 'Strain_33')
        self.addScalarField(field[3,:,:,:], 'Strain_23')
        self.addScalarField(field[4,:,:,:], 'Strain_13')
        self.addScalarField(field[5,:,:,:], 'Strain_12')
    
    def addFlux(self, field):
        self.addScalarField(field[0,:,:,:], 'flux_x')
        self.addScalarField(field[1,:,:,:], 'flux_y')
        self.addScalarField(field[2,:,:,:], 'flux_z')
        
    def addGradient(self, field):
        self.addScalarField(field[0,:,:,:], 'grad_x')
        self.addScalarField(field[1,:,:,:], 'grad_y')
        self.addScalarField(field[2,:,:,:], 'grad_z')
    
    def write(self):
        import os
        outputfilename = os.path.join(self.folder, 'output')
        if len(self.myCellData)>0:
            from pyevtk.hl import imageToVTK
            imageToVTK(outputfilename, cellData = self.myCellData)
            print(f'Output file written to {outputfilename}')