#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Feb 18 15:41:10 2022

@author: felix
"""
import cython
import numpy as np
cimport numpy as np
from libc.math cimport sqrt

@cython.boundscheck(False)
@cython.wraparound(False)
@cython.cdivision(True)
cdef void outerSelf(double* vector, complex* matrix):
    cdef int i,j
    for i in range(3):
        for j in range(3):
            matrix[i+j*3] = vector[i]*vector[j]

@cython.boundscheck(False)
@cython.wraparound(False)
@cython.cdivision(True)
def applyMoulinecSuquetNLO(np.ndarray[long,ndim=1]N_shape,np.ndarray[complex, ndim=4] primaryField, np.ndarray[double,ndim=1] freqX, np.ndarray[double,ndim=1] freqY, np.ndarray[double,ndim=1] freqZ, double mu):
    cdef int i,j,k,s,t
    cdef double [3] xi
    cdef complex [3] f,u
    
    cdef complex [6] tau_voigt
    cdef int Nx = len(freqX)
    cdef int Ny = len(freqY)
    cdef int Nz = len(freqZ)
    cdef double normXiSquared
    cdef complex xi_times_f
    
    for i in range(Nx):
        for j in range(Ny):
            for k in range(Nz):
                
                if i==0 and j == 0 and k==0:
                    pass
                
                elif (N_shape[0]%2==0 and i==N_shape[0]//2) or (N_shape[1]%2==0 and j==N_shape[1]//2) or (N_shape[2]%2==0 and k==N_shape[2]//2):
                    for s in range(6):
                        primaryField[s,i,j,k] = 0. 
                        
                else:

                    xi[0] = freqX[i]
                    xi[1] = freqY[j]
                    xi[2] = freqZ[k]
                    
                    normXiSquared = xi[0]*xi[0] + xi[1]*xi[1] + xi[2]*xi[2]
                    normXiSquared = sqrt(normXiSquared)
                    
                    xi[0]/=normXiSquared
                    xi[1]/=normXiSquared
                    xi[2]/=normXiSquared
                    
                    for s in range(6):
                        
                        tau_voigt[s] = primaryField[s,i,j,k]
                    
                    f[0] =  tau_voigt[0] * xi[0] + tau_voigt[5] * xi[1]  + tau_voigt[4] * xi[2]
                    f[1] =  tau_voigt[5] * xi[0] + tau_voigt[1] * xi[1]  + tau_voigt[3] * xi[2]
                    f[2] =  tau_voigt[4] * xi[0] + tau_voigt[3] * xi[1]  + tau_voigt[2] * xi[2]
                    
                    xi_times_f = f[0]*xi[0] + f[1]*xi[1] + f[2]*xi[2]
      
                    u[0] = (-f[0] + 0.5*xi_times_f*xi[0])/mu
                    u[1] = (-f[1] + 0.5*xi_times_f*xi[1])/mu
                    u[2] = (-f[2] + 0.5*xi_times_f*xi[2])/mu
                    
                    primaryField[0,i,j,k] = u[0]*xi[0]
                    primaryField[1,i,j,k] = u[1]*xi[1]
                    primaryField[2,i,j,k] = u[2]*xi[2]
                    
                    primaryField[3,i,j,k] = u[2]*xi[1] + u[1]*xi[2]
                    primaryField[4,i,j,k] = u[2]*xi[0] + u[0]*xi[2]
                    primaryField[5,i,j,k] = u[0]*xi[1] + u[1]*xi[0]

@cython.boundscheck(False)
@cython.wraparound(False)
@cython.cdivision(True)
def applyStaggeredGridNLO(np.ndarray[long,ndim=1]N_shape,np.ndarray[complex, ndim=4] primaryField, np.ndarray[complex,ndim=1] kk1, np.ndarray[complex,ndim=1] kk2, np.ndarray[complex,ndim=1] kk3, double mu):
    cdef int i,j,k,s,t
    cdef complex [3] f,u,kk
    cdef complex [6] tau_voigt
    cdef int Nx = len(kk1)
    cdef int Ny = len(kk2)
    cdef int Nz = len(kk3)
    cdef double normkSquared
    cdef complex k_bar_times_f
    for i in range(Nx):
        for j in range(Ny):
            for k in range(Nz):
                
                if i==0 and j == 0 and k==0:
                    pass
                
                else:

                    kk[0] = kk1[i]
                    kk[1] = kk2[j]
                    kk[2] = kk3[k]
                    
                    normkSquared = (kk[0]*kk[0].conjugate() + kk[1]*kk[1].conjugate() +kk[2]*kk[2].conjugate()    ).real               
                    normkSquared = sqrt(normkSquared)
                    kk[0] /=normkSquared
                    kk[1] /=normkSquared
                    kk[2] /=normkSquared
                    
                    for s in range(6):
                        tau_voigt[s] = primaryField[s,i,j,k]
                    
                    f[0] = -tau_voigt[0] * kk[0]             + tau_voigt[5] * kk[1].conjugate()   + tau_voigt[4] * kk[2].conjugate()
                    f[1] =  tau_voigt[5] * kk[0].conjugate() - tau_voigt[1] * kk[1]               + tau_voigt[3] * kk[2].conjugate()
                    f[2] =  tau_voigt[4] * kk[0].conjugate() + tau_voigt[3] * kk[1].conjugate()   - tau_voigt[2] * kk[2]

                    k_bar_times_f = kk[0].conjugate()*f[0] + kk[1].conjugate()*f[1] + kk[2].conjugate()*f[2]                    
                    
                    u[0] = (-f[0] + 0.5*k_bar_times_f *kk[0])/mu
                    u[1] = (-f[1] + 0.5*k_bar_times_f *kk[1])/mu
                    u[2] = (-f[2] + 0.5*k_bar_times_f *kk[2])/mu
                    
                                        
                    primaryField[0,i,j,k] = - u[0] * kk[0].conjugate()
                    primaryField[1,i,j,k] = - u[1] * kk[1].conjugate()
                    primaryField[2,i,j,k] = - u[2] * kk[2].conjugate()
                    
                    primaryField[3,i,j,k] = u[2]*kk[1] + u[1]*kk[2]
                    primaryField[4,i,j,k] = u[2]*kk[0] + u[0]*kk[2]
                    primaryField[5,i,j,k] = u[0]*kk[1] + u[1]*kk[0]

@cython.boundscheck(False)
@cython.wraparound(False)
@cython.cdivision(True)
def applyRotatedStaggeredGridNLO(np.ndarray[long,ndim=1]N_shape, np.ndarray[complex, ndim=4] primaryField, np.ndarray[complex,ndim=1] exp1, np.ndarray[complex,ndim=1] exp2, np.ndarray[complex,ndim=1] exp3, double mu):
    cdef int i,j,k,s,t
    cdef complex [3] f,u,kk
    cdef complex [6] tau_voigt
    cdef int Nx = len(exp1)
    cdef int Ny = len(exp2)
    cdef int Nz = len(exp3)
    cdef double normkSquared
    cdef complex k_bar_times_f
    for i in range(Nx):
        for j in range(Ny):
            for k in range(Nz):
                
                if i==0 and j == 0 and k==0:
                    pass
                
                elif (N_shape[0]%2==0 and i==N_shape[0]//2) or (N_shape[1]%2==0 and j==N_shape[1]//2) or (N_shape[2]%2==0 and k==N_shape[2]//2):
                    for s in range(6):
                        primaryField[s,i,j,k] = 0. 
                        
                else:

                    kk[0] = 0.25*(exp1[i]-1.)*(exp2[j]+1.)*(exp3[k]+1.)
                    kk[1] = 0.25*(exp1[i]+1.)*(exp2[j]-1.)*(exp3[k]+1.)
                    kk[2] = 0.25*(exp1[i]+1.)*(exp2[j]+1.)*(exp3[k]-1.)
                    
                    normkSquared = (kk[0]*kk[0].conjugate() + kk[1]*kk[1].conjugate() +kk[2]*kk[2].conjugate()    ).real               
                    normkSquared = sqrt(normkSquared)
                    kk[0] /=normkSquared
                    kk[1] /=normkSquared
                    kk[2] /=normkSquared
                    
                    for s in range(6):
                        
                        tau_voigt[s] = primaryField[s,i,j,k]
                    
                    f[0] =  tau_voigt[0] * kk[0].conjugate() + tau_voigt[5] * kk[1].conjugate()   + tau_voigt[4] * kk[2].conjugate()
                    f[1] =  tau_voigt[5] * kk[0].conjugate() + tau_voigt[1] * kk[1].conjugate()   + tau_voigt[3] * kk[2].conjugate()
                    f[2] =  tau_voigt[4] * kk[0].conjugate() + tau_voigt[3] * kk[1].conjugate()   + tau_voigt[2] * kk[2].conjugate()

                    k_bar_times_f = kk[0].conjugate()*f[0] + kk[1].conjugate()*f[1] + kk[2].conjugate()*f[2]                    
                    
                    u[0] = (-f[0] + 0.5*k_bar_times_f *kk[0])/mu
                    u[1] = (-f[1] + 0.5*k_bar_times_f *kk[1])/mu
                    u[2] = (-f[2] + 0.5*k_bar_times_f *kk[2])/mu
                    
                                        
                    primaryField[0,i,j,k] = u[0]*kk[0]
                    primaryField[1,i,j,k] = u[1]*kk[1]
                    primaryField[2,i,j,k] = u[2]*kk[2]
                    
                    primaryField[3,i,j,k] = u[2]*kk[1] + u[1]*kk[2]
                    primaryField[4,i,j,k] = u[2]*kk[0] + u[0]*kk[2]
                    primaryField[5,i,j,k] = u[0]*kk[1] + u[1]*kk[0]
                    
                    
                    
                    
def applyMoulinecSuquetNLO_HEAT(np.ndarray[long,ndim=1]N_shape, np.ndarray[complex, ndim=4] primaryField, np.ndarray[double,ndim=1] freqX, np.ndarray[double,ndim=1] freqY, np.ndarray[double,ndim=1] freqZ, double kappa):
    cdef int i,j,k
    cdef double [3] xi
    cdef double normXiSquared
    cdef complex f
    cdef int Nx = len(freqX)
    cdef int Ny = len(freqY)
    cdef int Nz = len(freqZ)
    
    for i in range(Nx):
        for j in range(Ny):
            for k in range(Nz):
                
                if i==0 and j == 0 and k==0:
                    pass
                
                elif (N_shape[0]%2==0 and i==N_shape[0]//2) or (N_shape[1]%2==0 and j==N_shape[1]//2) or (N_shape[2]%2==0 and k==N_shape[2]//2):
                    primaryField[0,i,j,k] = 0
                    primaryField[1,i,j,k] = 0.
                    primaryField[2,i,j,k] = 0.
                    
                else:
                    xi[0] = freqX[i]
                    xi[1] = freqY[j]
                    xi[2] = freqZ[k]
                    
                    normXiSquared = xi[0]*xi[0] + xi[1]*xi[1] + xi[2]*xi[2]
                    normXiSquared = sqrt(normXiSquared)
                    xi[0] /= normXiSquared
                    xi[1] /= normXiSquared
                    xi[2] /= normXiSquared
                    
                    f = (xi[0] * primaryField[0,i,j,k] + xi[1] * primaryField[1,i,j,k] + xi[2] * primaryField[2,i,j,k])/kappa
                    
                    primaryField[0,i,j,k] = -f * xi[0]
                    primaryField[1,i,j,k] = -f * xi[1]
                    primaryField[2,i,j,k] = -f * xi[2]